@@ -1,17 +1,27 @@
 /****************************************************************************
- *  ircconnection.cpp
- *
- *  Copyright (c) 2010 by Prokhin Alexey <alexey.prokhin@yandex.ru>
- *
- ***************************************************************************
- *                                                                         *
- *   This library is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************
- *****************************************************************************/
+**
+** qutIM - instant messenger
+**
+** Copyright (C) 2011 Prokhin Alexey <alexey.prokhin@yandex.ru>
+**
+*****************************************************************************
+**
+** $QUTIM_BEGIN_LICENSE$
+** This program is free software: you can redistribute it and/or modify
+** it under the terms of the GNU General Public License as published by
+** the Free Software Foundation, either version 3 of the License, or
+** (at your option) any later version.
+**
+** This program is distributed in the hope that it will be useful,
+** but WITHOUT ANY WARRANTY; without even the implied warranty of
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU General Public License for more details.
+**
+** You should have received a copy of the GNU General Public License
+** along with this program.  If not, see http://www.gnu.org/licenses/.
+** $QUTIM_END_LICENSE$
+**
+****************************************************************************/
 
 #include "ircconnection.h"
 #include "ircaccount_p.h"
@@ -20,6 +30,8 @@
 #include "irccontact_p.h"
 #include "ircchannelparticipant.h"
 #include "ircavatar.h"
+#include "ircwhoisreplieshandler.h"
+#include "ircstandartctpchandler.h"
 #include <QHostInfo>
 #include <QTextCodec>
 #include <QRegExp>
@@ -28,45 +40,34 @@
 #include <qutim/messagesession.h>
 #include <qutim/networkproxy.h>
 #include <qutim/dataforms.h>
-#include <qutim/notificationslayer.h>
+#include <qutim/notification.h>
+#include <qutim/passworddialog.h>
 
 namespace qutim_sdk_0_3 {
 
 namespace irc {
 
 static QRegExp ctpcRx("^\\001(\\S+)( (.*)|)\\001");
-QMultiHash<QString, IrcCommandAlias*> IrcConnection::m_aliases;
-
-IrcPingAlias::IrcPingAlias() :
-	IrcCommandAlias("ping", QString())
-{
-}
-
-QString IrcPingAlias::generate(IrcCommandAlias::Type aliasType, const QStringList &params,
-							   const QHash<QChar, QString> &extParams, QString *error) const
-{
-	Q_UNUSED(aliasType);
-	Q_UNUSED(error);
-	Q_UNUSED(extParams);
-	QDateTime current = QDateTime::currentDateTime();
-	QString timeStamp = QString("%1.%2").arg(current.toTime_t()).arg(current.time().msec());
-	return QString("PRIVMSG %1 :\001PING %2\001").arg(params.value(0)).arg(timeStamp);
-}
 
 IrcConnection::IrcConnection(IrcAccount *account, QObject *parent) :
 	QObject(parent), m_hostLookupId(0)
 {
-	m_socket = new QTcpSocket(this);
+	m_socket = new QSslSocket(this);
 	m_socket->setProxy(NetworkProxyManager::toNetworkProxy(NetworkProxyManager::settings(account)));
 	m_account = account;
+	m_messagesTimer.setInterval(500);
+	connect(&m_messagesTimer, SIGNAL(timeout()), SLOT(sendNextMessage()));
 	connect(m_socket, SIGNAL(readyRead()), SLOT(readData()));
 	connect(m_socket, SIGNAL(stateChanged(QAbstractSocket::SocketState)), SLOT(stateChanged(QAbstractSocket::SocketState)));
 	connect(m_socket, SIGNAL(error(QAbstractSocket::SocketError)), SLOT(error(QAbstractSocket::SocketError)));
+	connect(m_socket, SIGNAL(sslErrors(QList<QSslError>)), SLOT(sslErrors(QList<QSslError>)));
+	connect(m_socket, SIGNAL(encrypted()), SLOT(encrypted()));
 	// Register handlers
 	foreach(const ObjectGenerator *gen, ObjectGenerator::module<IrcServerMessageHandler>())
 		registerHandler(gen->generate<IrcServerMessageHandler>());
 	foreach(const ObjectGenerator *gen, ObjectGenerator::module<IrcCtpcHandler>())
 		registerCtpcHandler(gen->generate<IrcCtpcHandler>());
+	registerHandler(new IrcWhoisRepliesHandler(this));
 
 	m_cmds
 		<< 432  // ERR_ERRONEUSNICKNAME
@@ -100,42 +101,11 @@ IrcConnection::IrcConnection(IrcAccount *account, QObject *parent) :
 		<< 263  // RPL_TRYAGAIN
 		<< 301  // RPL_AWAY
 		<< 305  // RPL_UNAWAY
-		<< 306; // RPL_NOWAWAY
+		<< 306;  // RPL_NOWAWAY
 	registerHandler(this);
 
-	m_ctpcCmds << "PING" << "ACTION" << "CLIENTINFO" << "VERSION" << "TIME";
-	registerCtpcHandler(this);
+	registerCtpcHandler(new IrcStandartCtpcHandler(this));
 	registerCtpcHandler(IrcAvatar::instance());
-
-	static bool init = false;
-	if (!init) {
-		IrcAccount::registerLogMsgColor("ERROR", "red");
-		IrcAccount::registerLogMsgColor("Notice", "magenta");
-		IrcAccount::registerLogMsgColor("MOTD", "green");
-		IrcAccount::registerLogMsgColor("Welcome", "green");
-		IrcAccount::registerLogMsgColor("Support", "green");
-		IrcAccount::registerLogMsgColor("Users", "green");
-		IrcAccount::registerLogMsgColor("Away", "green");
-
-		registerAlias(new IrcCommandAlias("ctpc", "PRIVMSG %1 :\001%2-\001"));
-		registerAlias(new IrcCommandAlias("me", "PRIVMSG %1 :\001ACTION %2-\001",
-									  IrcCommandAlias::Console));
-		registerAlias(new IrcCommandAlias("me", "PRIVMSG %n :\001ACTION %0\001",
-									  IrcCommandAlias::Channel | IrcCommandAlias::PrivateChat));
-		registerAlias(new IrcCommandAlias("cs", "PRIVMSG ChanServ :%0"));
-		registerAlias(new IrcCommandAlias("ns", "PRIVMSG NickServ :%0"));
-		registerAlias(new IrcCommandAlias("kick", "KICK %n %0",
-									  IrcCommandAlias::Channel));
-		registerAlias(new IrcCommandAlias("ban", "MODE %n +b %0",
-									  IrcCommandAlias::Channel));
-		registerAlias(new IrcCommandAlias("msg", "PRIVMSG %0"));
-		registerAlias(new IrcCommandAlias("clientinfo", "PRIVMSG %1 :\001CLIENTINFO\001"));
-		registerAlias(new IrcCommandAlias("version", "PRIVMSG %1 :\001VERSION\001"));
-		registerAlias(new IrcCommandAlias("time", "PRIVMSG %1 :\001TIME\001"));
-		registerAlias(new IrcCommandAlias("avatar", "PRIVMSG %1 :\001AVATAR\001"));
-		registerAlias(new IrcPingAlias);
-		init = true;
-	}
 }
 
 IrcConnection::~IrcConnection()
@@ -172,8 +142,15 @@ void IrcConnection::handleMessage(IrcAccount *account, const QString &name,  con
 		if (status == Status::Connecting) {
 			account->setStatus(Status::Online);
 			foreach (IrcChannel *channel, account->d->channels) {
-				if (channel->d->autojoin)
+				if (channel->d->autojoin || channel->d->reconnect) {
 					channel->join();
+					channel->d->reconnect = false;
+				}
+			}
+			if (!m_nickPassword.isEmpty()) {
+				send(QString("PRIVMSG NickServ :IDENTIFY %1 %2")
+					 .arg(m_nicks.value(0))
+					 .arg(m_nickPassword));
 			}
 		}
 		QString msg;
@@ -216,7 +193,7 @@ void IrcConnection::handleMessage(IrcAccount *account, const QString &name,  con
 				debug() << "Unknown CTPC request" << ctpcCmd << "from" << name;
 			return;
 		}
-		handleTextMessage(name, params.value(0),params.value(1));
+		handleTextMessage(name, host, params.value(0), params.value(1));
 	} else if (cmd == "JOIN") {
 		QString channelName = params.value(0);
 		// Create a new IrcChannel if we have joined a channel.
@@ -247,13 +224,15 @@ void IrcConnection::handleMessage(IrcAccount *account, const QString &name,  con
 		IrcContact *contact = account->getContact(name, false);
 		if (contact) {
 			emit contact->quit(params.value(0));
-			contact->deleteLater();
 		} else {
 			debug() << "QUIT message from the unknown contact" << name;
 		}
 	} else if (cmd == "ERROR") {
 		m_account->log(params.value(0), false, "ERROR");
-		Notifications::send(Notifications::System, m_account, params.value(0));
+		NotificationRequest request(Notification::System);
+		request.setObject(m_account);
+		request.setText(params.value(0));
+		request.send();
 		m_account->setStatus(Status(Status::Offline));
 	} else if (cmd == 332) { // RPL_TOPIC
 		IrcChannel *channel = account->getChannel(params.value(1), false);
@@ -300,50 +279,54 @@ void IrcConnection::handleMessage(IrcAccount *account, const QString &name,  con
 		m_account->log(tr("End of message of the day"), false, "MOTD");
 	} else if (cmd == 372) { // RPL_MOTD
 		m_account->log(params.value(1), false, "MOTD");
+
 	} else if (cmd == 321) { // RPL_LISTSTART
 		if (m_account->d->channelListForm)
-			m_account->d->channelListForm->listStarted();
+			m_account->d->channelListForm.data()->listStarted();
 		else
-			m_account->log(tr("End of /LIST"), true, "LIST");
+			m_account->log(tr("Start of /LIST"), m_account->isUserInputtedCommand("LIST"), "LIST");
 	} else if (cmd == 322) { // RPL_LIST
 		QString channel = params.value(1);
 		QString users = params.value(2);
 		QString topic = IrcProtocol::ircFormatToHtml(params.value(3));
 		if (m_account->d->channelListForm)
-			m_account->d->channelListForm->addChannel(channel, users, topic);
+			m_account->d->channelListForm.data()->addChannel(channel, users, topic);
 		else
 			m_account->log(QString("Channel: %1 Users: %2 Topic: %3")
 						   .arg(channel)
 						   .arg(users)
 						   .arg(topic),
-						   true,
+						   m_account->isUserInputtedCommand("LIST"),
 						   "LIST");
 	} else if (cmd == 323) { // RPL_LISTEND
 		if (m_account->d->channelListForm)
-			m_account->d->channelListForm->listEnded();
+			m_account->d->channelListForm.data()->listEnded();
 		else
-			m_account->log(tr("End of /LIST"), true, "LIST");
+			m_account->log(tr("End of /LIST"), m_account->isUserInputtedCommand("LIST", true), "LIST");
 	} else if (cmd == 521) { // ERR_LISTSYNTAX
 		QString error = tr("Bad list syntax, type /QUOTE HELP LIST");
 		if (m_account->d->channelListForm)
-			m_account->d->channelListForm->error(error);
+			m_account->d->channelListForm.data()->error(error);
 		m_account->log(error, true, "ERROR");
 	} else if (cmd == 263) { // RPL_TRYAGAIN
 		QString error = tr("Server load is temporarily too heavy.\nPlease wait a while and try again.");
 		if (m_account->d->channelListForm)
-			m_account->d->channelListForm->error(error);
+			m_account->d->channelListForm.data()->error(error);
 		m_account->log(error, true, "ERROR");
 	} else if (cmd == 301) { // RPL_AWAY
-		IrcContact *contact = account->getContact(name, false);
+		QString nick = params.value(1);
+		IrcContact *contact = account->getContact(nick, false);
 		if (contact) {
-			QString awayMsg = params.value(0);
+			QString awayMsg = params.value(2);
 			contact->setAway(awayMsg);
 			if (!awayMsg.isEmpty())
-				m_account->log(tr("%1 set away message %2").arg(name).arg(awayMsg), false, "Away");
+				m_account->log(tr("%1 set away message \"%2\"").arg(nick).arg(awayMsg),
+							   m_account->isUserInputtedCommand("WHOIS"),
+							   "Away");
 			else
-				m_account->log(tr("%1 removed away message").arg(name), false, "Away");
-		} else {
-			debug() << "AWAY message from the unknown contact" << name;
+				m_account->log(tr("%1 removed away message").arg(nick),
+							   m_account->isUserInputtedCommand("WHOIS"),
+							   "Away");
 		}
 	} else if (cmd == 305) { // RPL_UNAWAY
 		m_account->log(tr("You are no longer marked as being away"), false, "Away");
@@ -352,134 +335,33 @@ void IrcConnection::handleMessage(IrcAccount *account, const QString &name,  con
 	}
 }
 
-void IrcConnection::handleCtpcRequest(IrcAccount *account, const QString &sender, const QString &senderHost,
-									  const QString &receiver, const QString &cmd, const QString &params)
+void IrcConnection::send(QString command, bool highPriority)
 {
-	Q_UNUSED(account);
-	Q_UNUSED(senderHost);
-	Q_UNUSED(receiver);
-	if (cmd == "PING") {
-		sendCtpcReply(sender, "PING", params);
-	} else if (cmd == "ACTION") {
-		handleTextMessage(sender, receiver, QLatin1String("/me ") + params);
-	} else if (cmd == "CLIENTINFO") {
-		QStringList tags = m_ctpcHandlers.keys();
-		QString params = QString("IRC plugin for qutIM %1 - http://qutim.org - Supported tags: %2")
-						 .arg(qutimVersionStr()).arg(tags.join(","));
-		sendCtpcReply(sender, "CLIENTINFO", params);
-	} else if (cmd == "VERSION") {
-		QString params = QString("IRC plugin %1, qutim %2")
-						 .arg(qutimIrcVersionStr())
-						 .arg(qutimVersionStr());
-		sendCtpcReply(sender, "VERSION", params);
-	} else if (cmd == "TIME") {
-		sendCtpcReply(sender, "TIME", QDateTime::currentDateTime().toString("ddd MMM dd hh:mm:ss yyyy"));
+	if (!command.isEmpty()) {
+		if (highPriority)
+			m_messagesQueue.push_back(command);
+		else
+			m_lowPriorityMessagesQueue.push_back(command);
+		if (!m_messagesTimer.isActive())
+			m_messagesTimer.start();
+		sendNextMessage();
 	}
 }
 
-void IrcConnection::handleCtpcResponse(IrcAccount *account, const QString &sender, const QString &senderHost,
-									   const QString &receiver, const QString &cmd, const QString &params)
-{
-	Q_UNUSED(account);
-	Q_UNUSED(senderHost);
-	Q_UNUSED(receiver);
-	if (cmd == "PING") {
-		QDateTime current = QDateTime::currentDateTime();
-		double receivedStamp = params.toDouble();
-		if (receivedStamp >= 0) {
-			double currentStamp = (double)current.time().msec() / 1000 + current.toTime_t();
-			double diff = currentStamp - receivedStamp;
-			account->log(tr("Received CTCP-PING reply from %1: %2 seconds", "", diff)
-						 .arg(sender)
-						 .arg(diff, 0, 'f', 3),
-						 true, "CTPC");
-		}
-	} else if (cmd == "CLIENTINFO" || cmd == "VERSION" || "TIME") {
-		account->log(tr("Received CTCP-%1 reply from %2: %3")
-					 .arg(cmd)
-					 .arg(sender)
-					 .arg(params),
-					 true, "CTPC");
-	}
-}
-
-void IrcConnection::removeAlias(const QString &name)
-{
-	qDeleteAll(m_aliases.values(name));
-	m_aliases.remove(name);
-}
-
-void IrcConnection::removeAlias(IrcCommandAlias *alias)
-{
-	QHash<QString, IrcCommandAlias*>::iterator itr = m_aliases.begin();
-	QHash<QString, IrcCommandAlias*>::iterator endItr = m_aliases.end();
-	while (itr != endItr) {
-		if (*itr == alias) {
-			delete alias;
-			m_aliases.erase(itr);
-			return;
-		}
-		++itr;
-	}
-}
-
-void IrcConnection::send(QString command, IrcCommandAlias::Type aliasType, const ExtendedParams &extParams) const
-{
-	if (aliasType != IrcCommandAlias::Disabled) {
-		bool found;
-		QString lastCmdName;
-		QString cmdName; // Current command name
-		QString cmdParamsStr; // Current parameters
-		int i = 0;
-		for (; i < 10; ++i) {
-			found = false;			
-			cmdName = command.mid(0, command.indexOf(' '));
-			if (cmdName.compare(lastCmdName, Qt::CaseInsensitive) == 0) // To avoid recursion
-				break;
-			cmdParamsStr = command.mid(cmdName.length() + 1);
-			QStringList params; // Parameters from the command line
-			params << cmdParamsStr;
-			params += cmdParamsStr.split(' ', QString::SkipEmptyParts);
-			foreach (IrcCommandAlias *alias, m_aliases) {
-				if (cmdName.compare(alias->name(), Qt::CaseInsensitive) == 0) {
-					QString error;
-					QString newCommand = alias->generate(aliasType, params, extParams, &error);
-					if (!error.isEmpty()) {
-						m_account->log(error, true, "ERROR");
-						return;
-					} else if (!newCommand.isEmpty()) {
-						command = newCommand;
-						lastCmdName = cmdName;
-						found = true;
-					}
-					break;
-				}
-			}
-			if (!found)
-				break;
-		}
-		if (i == 0 && !found) // A suitable alias has not found
-			command = cmdName.toUpper() + " " + cmdParamsStr;
-	}
-	QByteArray data = m_codec->fromUnicode(command) + "\r\n";
-	debug(VeryVerbose) << ">>>>" << data.trimmed();
-	m_socket->write(data);
-}
-
-void IrcConnection::sendCtpcRequest(const QString &contact, const QString &cmd, const QString &params)
+void IrcConnection::sendCtpcRequest(const QString &contact, const QString &cmd, const QString &params, bool highPriority)
 {
 	QString ctpc(cmd);
 	if (!params.isEmpty())
 		ctpc += " " + params;
-	send(QString("PRIVMSG %1 :\001%2\001").arg(contact).arg(ctpc));
+	send(QString("PRIVMSG %1 :\001%2\001").arg(contact).arg(ctpc), highPriority);
 }
 
-void IrcConnection::sendCtpcReply(const QString &contact, const QString &cmd, const QString &params)
+void IrcConnection::sendCtpcReply(const QString &contact, const QString &cmd, const QString &params, bool highPriority)
 {
 	QString ctpc(cmd);
 	if (!params.isEmpty())
 		ctpc += " " + params;
-	send(QString("NOTICE %1 :\001%2\001").arg(contact).arg(ctpc));
+	send(QString("NOTICE %1 :\001%2\001").arg(contact).arg(ctpc), highPriority);
 }
 
 void IrcConnection::disconnectFromHost(bool force)
@@ -489,15 +371,14 @@ void IrcConnection::disconnectFromHost(bool force)
 		m_hostLookupId = 0;
 	}
 	if (m_socket->state() != QTcpSocket::UnconnectedState) {
-		if (force || m_socket->state() != QTcpSocket::ConnectedState)
-			m_socket->disconnectFromHost();
-		else
+		if (!force && m_socket->state() == QTcpSocket::ConnectedState)
 			send(QString("QUIT :%1").arg("qutIM: IRC plugin"));
+		m_socket->disconnectFromHost();
 	}
 	foreach (IrcChannel *channel, m_account->d->channels) {
 		if (channel->isJoined()) {
 			channel->leave(true);
-			channel->setAutoJoin();
+			channel->d->reconnect = true;
 		}
 	}
 }
@@ -512,14 +393,14 @@ void IrcConnection::loadSettings()
 	m_servers.clear();
 	m_currentServer = 0;
 	Config cfg = m_account->config();
-	Config protoCfg = m_account->protocol()->config();
 	// List of servers.
 	cfg.beginArray("servers");
 	for (int i = 0; i < cfg.arraySize(); ++i) {
 		cfg.setArrayIndex(i);
 		IrcServer server;
 		server.hostName = cfg.value("hostName", QString());
-		server.port = cfg.value("port", 6667);
+		server.ssl = cfg.value("ssl", false);
+		server.port = cfg.value("port", server.ssl ? 6667 : 6697);
 		server.protectedByPassword = cfg.value("protectedByPassword", false);
 		if (server.protectedByPassword)
 			server.password = cfg.value("password", QString(), Config::Crypted);
@@ -528,18 +409,20 @@ void IrcConnection::loadSettings()
 	cfg.endArray();
 	// User nickname
 	m_nicks = cfg.value("nicks").toStringList();
-	if (m_nicks.isEmpty())
-		m_nicks = protoCfg.value("nicks").toStringList();
 	if (m_nick.isEmpty())
 		m_nick = m_nicks.value(0);
 	// User fullname.
 	m_fullName = cfg.value("fullName").toString();
-	if (m_fullName.isEmpty())
-		m_fullName = protoCfg.value("fullName").toString();
+	m_nickPassword = cfg.value("nickPassword").toString();
 	m_codec = QTextCodec::codecForName(cfg.value("codec", "utf8").toLatin1());
 	if (!m_codec)
 		m_codec = QTextCodec::codecForName("utf8");
 	Q_ASSERT(m_codec);
+#ifndef QUTIM_MOBILE_UI
+	m_autoRequestWhois = cfg.value("autoRequestWhois", true);
+#else
+	m_autoRequestWhois = cfg.value("autoRequestWhois", false);
+#endif
 }
 
 void IrcConnection::tryConnectToNextServer()
@@ -555,12 +438,18 @@ void IrcConnection::tryConnectToNextServer()
 	if (!error.isEmpty()) {
 		debug() << error;
 		m_account->setStatus(Status::Offline);
-		Notifications::send(Notifications::System, m_account, error);
+		NotificationRequest request(Notification::System);
+		request.setObject(m_account);
+		request.setText(error);
+		request.send();
 		return;
 	}
 	m_currentNick = -1;
 	IrcServer server = m_servers.at(m_currentServer);
-	m_hostLookupId = QHostInfo::lookupHost(server.hostName, this, SLOT(hostFound(QHostInfo)));
+	if (server.ssl)
+		m_socket->connectToHostEncrypted(server.hostName, server.port);
+	else
+		m_hostLookupId = QHostInfo::lookupHost(server.hostName, this, SLOT(hostFound(QHostInfo)));
 }
 
 void IrcConnection::hostFound(const QHostInfo &host)
@@ -580,17 +469,43 @@ void IrcConnection::tryNextNick()
 		debug() << "Set at least one nick before connecting";
 		return;
 	}
-	if (++m_currentNick >= m_nicks.size())
+	if (++m_currentNick >= m_nicks.size()) {
 		tryConnectToNextServer(); // TODO: Or should we stop connecting?
+		return;
+	}
 	m_nick = m_nicks.at(m_currentNick);
-	send(QString("NICK %1").arg(m_nick));
-	send(QString("USER %1 %2 * :%3").arg(m_nick).arg(0).arg(m_fullName.isEmpty() ? m_nick : m_fullName));
+	send(QString("NICK %1\nUSER %1 %2 * :%3")
+		 .arg(m_nick).arg(0)
+		 .arg(m_fullName.isEmpty() ? m_nick : m_fullName));
 }
 
-void IrcConnection::handleTextMessage(const QString &who, const QString &to, const QString &text)
+void IrcConnection::sendNextMessage()
 {
-	QString plainText;
-	QString html = IrcProtocol::ircFormatToHtml(text, &plainText);
+	uint currentTime = QDateTime::currentDateTime().toTime_t();
+	if (currentTime - m_lastMessageTime <= 2)
+		return;
+
+	QString command;
+	if (!m_messagesQueue.isEmpty())
+		command = m_messagesQueue.takeFirst();
+	else if (!m_lowPriorityMessagesQueue.isEmpty())
+		command = m_lowPriorityMessagesQueue.takeFirst();
+	else
+		return;
+
+	QByteArray data = m_codec->fromUnicode(command) + "\r\n";
+	debug(VeryVerbose) << ">>>>" << data.trimmed();
+	m_socket->write(data);
+
+	m_lastMessageTime = QDateTime::currentDateTime().toTime_t();
+	if (m_messagesQueue.isEmpty() && m_lowPriorityMessagesQueue.isEmpty())
+		m_messagesTimer.stop();
+}
+
+void IrcConnection::handleTextMessage(const QString &from, const QString &fromHost, const QString &to, const QString &text)
+{
+	QString plainText = IrcProtocol::ircFormatToPlainText(text);
+	QString html = IrcProtocol::ircFormatToHtml(text);
 	bool isPrivate = (to == m_nick);
 	Message msg(plainText);
 	msg.setIncoming(true);
@@ -598,7 +513,7 @@ void IrcConnection::handleTextMessage(const QString &who, const QString &to, con
 	msg.setProperty("html", html);
 	ChatSession *session;
 	if (isPrivate) {
-		IrcContact *contact = m_account->getContact(who, true);
+		IrcContact *contact = m_account->getContact(from, fromHost, true);
 		msg.setChatUnit(contact);
 		session = ChatLayer::instance()->getSession(contact, true);
 		connect(session, SIGNAL(destroyed()), contact, SLOT(onSessionDestroyed()));
@@ -610,10 +525,8 @@ void IrcConnection::handleTextMessage(const QString &who, const QString &to, con
 		}
 		session = ChatLayer::instance()->getSession(channel, true);
 		msg.setChatUnit(channel);
-		msg.setProperty("senderName", who);
-		msg.setProperty("senderId", who);
-		if (!text.contains(m_nick))
-			msg.setProperty("silent", true);
+		msg.setProperty("senderName", from);
+		msg.setProperty("senderId", from);
 	}
 	session->appendMessage(msg);
 }
@@ -686,11 +599,23 @@ void IrcConnection::stateChanged(QAbstractSocket::SocketState state)
 	if (state == QAbstractSocket::ConnectedState) {
 		IrcServer server = m_servers.at(m_currentServer);
 		if (server.protectedByPassword) {
-			send(QString("PASS %1").arg(server.password));
-		} else {
-			tryNextNick();
+			if (m_passDialog) {
+				return;
+			} else if (server.password.isEmpty()) {
+				m_passDialog = PasswordDialog::request(m_account);
+				QObject::connect(m_passDialog.data(), SIGNAL(entered(QString,bool)),
+								 this, SLOT(passwordEntered(QString,bool)));
+				QObject::connect(m_passDialog.data(), SIGNAL(rejected()),
+								 m_passDialog.data(), SLOT(deleteLater()));
+				return;
+			} else {
+				send(QString("PASS %1").arg(server.password));
+			}
 		}
+		tryNextNick();
 	} else if (state == QAbstractSocket::UnconnectedState) {
+		if (m_passDialog)
+			m_passDialog.data()->deleteLater();
 		m_account->setStatus(Status::Offline);
 	}
 }
@@ -698,8 +623,48 @@ void IrcConnection::stateChanged(QAbstractSocket::SocketState state)
 void IrcConnection::error(QAbstractSocket::SocketError error)
 {
 	debug() << "Connection error:" << error;
-	Notifications::send(Notifications::System, m_account,
-						tr("Network error: %1").arg(m_socket->errorString()));
+	NotificationRequest request(Notification::System);
+	request.setObject(m_account);
+	request.setText(tr("Network error: %1").arg(m_socket->errorString()));
+	request.send();
+	m_account->log(m_socket->errorString(), false, "ERROR");
+}
+
+void IrcConnection::sslErrors(const QList<QSslError> &errors)
+{
+	QString notification;
+	foreach (const QSslError &error, errors) {
+		m_account->log(error.errorString(), false, "ERROR");
+		if (!notification.isNull())
+			notification.append("\n");
+		notification.append(error.errorString());
+	}
+	NotificationRequest request(Notification::System);
+	request.setObject(m_account);
+	request.setText(tr("SSL error: %1").arg(notification));
+	request.send();
+}
+
+void IrcConnection::encrypted()
+{
+	m_account->log(tr("SSL handshake completed"), false, "Notice");
+}
+
+void IrcConnection::passwordEntered(const QString &password, bool remember)
+{
+	Q_ASSERT(sender() == m_passDialog.data());
+	if (remember) {
+		Config cfg = m_account->config();
+		cfg.beginArray("servers");
+		cfg.setArrayIndex(m_currentServer);
+		cfg.setValue("password", password, Config::Crypted);
+		cfg.endArray();
+		m_servers[m_currentServer].password = password;
+	}
+	m_passDialog.data()->deleteLater();
+	send(QString("PASS %1").arg(password));
+	tryNextNick();
 }
 
 } } // namespace qutim_sdk_0_3::irc
+
