@@ -1,25 +1,34 @@
 /****************************************************************************
- *  jmucsession.cpp
- *
- *  Copyright (c) 2009 by Nigmatullin Ruslan <euroelessar@gmail.com>
- *  Copyright (c) 2011 by Sidorov Aleksey <sauron@citadelspb.com>
- *
- ***************************************************************************
- *                                                                         *
- *   This library is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************
-*****************************************************************************/
+**
+** qutIM - instant messenger
+**
+** Copyright (C) 2011 Ruslan Nigmatullin <euroelessar@yandex.ru>
+** Copyright (C) 2011 Sidorov Aleksey <sauron@citadelspb.com>
+**
+*****************************************************************************
+**
+** $QUTIM_BEGIN_LICENSE$
+** This program is free software: you can redistribute it and/or modify
+** it under the terms of the GNU General Public License as published by
+** the Free Software Foundation, either version 3 of the License, or
+** (at your option) any later version.
+**
+** This program is distributed in the hope that it will be useful,
+** but WITHOUT ANY WARRANTY; without even the implied warranty of
+** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU General Public License for more details.
+**
+** You should have received a copy of the GNU General Public License
+** along with this program.  If not, see http://www.gnu.org/licenses/.
+** $QUTIM_END_LICENSE$
+**
+****************************************************************************/
 
 #include "jmucsession.h"
 #include "jmucuser.h"
 #include "../jaccount.h"
 #include "../roster/jmessagesession.h"
 #include "../roster/jmessagehandler.h"
-#include "../vcard/jvcardmanager.h"
 #include "../../jprotocol.h"
 #include "jbookmarkmanager.h"
 #include <qutim/message.h>
@@ -27,7 +36,7 @@
 #include <qutim/messagesession.h>
 #include <jreen/vcardupdate.h>
 #include <jreen/delayeddelivery.h>
-#include "jabber_global.h"
+#include "../../../jabber_global.h"
 #include <QStringBuilder>
 #include <QMessageBox>
 #include <QFile>
@@ -36,12 +45,12 @@
 #include "../roster/jcontactresource_p.h"
 #include <jreen/client.h>
 #include "../roster/jsoftwaredetection.h"
-#include <qutim/notificationslayer.h>
+#include <qutim/notification.h>
 #include <QApplication>
 #include <qutim/debug.h>
 #include <QInputDialog>
 
-using namespace jreen;
+using namespace Jreen;
 using namespace qutim_sdk_0_3;
 
 namespace Jabber
@@ -50,16 +59,15 @@ class JMUCSessionPrivate
 {
 public:
 	QPointer<JAccount> account;
-	jreen::MUCRoom *room;
-	jreen::JID jid;
-	QString nick;
+	Jreen::MUCRoom *room;
+	Jreen::JID jid;
 	QString title;
 	QString topic;
 	QHash<QString, quint64> messages;
 	QHash<QString, JMUCUser *> users;
 	bool isJoined;
 	bool isAutoRejoin;
-	jreen::Bookmark::Conference bookmark;
+	Jreen::Bookmark::Conference bookmark;
 	QPointer<JConferenceConfig> config;
 	bool avatarsAutoLoad;
 	bool isError;
@@ -67,28 +75,28 @@ public:
 	QString *thread;
 };
 
-JMUCSession::JMUCSession(const jreen::JID &room, const QString &password, JAccount *account) :
+JMUCSession::JMUCSession(const Jreen::JID &room, const QString &password, JAccount *account) :
 	Conference(account), d_ptr(new JMUCSessionPrivate)
 {
 	Q_D(JMUCSession);
 	d->jid = room.bareJID();
-	d->nick = room.resource();
 	d->account = account;
-	d->room = new jreen::MUCRoom(account->client(), room);
-	connect(d->room, SIGNAL(presenceReceived(jreen::Presence,const jreen::MUCRoom::Participant*)),
-			this, SLOT(onParticipantPresence(jreen::Presence,const jreen::MUCRoom::Participant*)));
-	connect(d->room, SIGNAL(presenceReceived(jreen::Presence,const jreen::MUCRoom::Participant*)),
-			d->account->softwareDetection(), SLOT(handlePresence(jreen::Presence)));
-	connect(d->room, SIGNAL(messageReceived(jreen::Message,bool)),
-			this, SLOT(onMessage(jreen::Message,bool)));
-	connect(d->room, SIGNAL(serviceMessageReceived(jreen::Message)),
-			this, SLOT(onServiceMessage(jreen::Message)));
+	d->room = new Jreen::MUCRoom(account->client(), room);
+	d->room->setPassword(password);
+	connect(d->room, SIGNAL(presenceReceived(Jreen::Presence,const Jreen::MUCRoom::Participant*)),
+			this, SLOT(onParticipantPresence(Jreen::Presence,const Jreen::MUCRoom::Participant*)));
+	connect(d->room, SIGNAL(presenceReceived(Jreen::Presence,const Jreen::MUCRoom::Participant*)),
+			d->account->softwareDetection(), SLOT(handlePresence(Jreen::Presence)));
+	connect(d->room, SIGNAL(messageReceived(Jreen::Message,bool)),
+			this, SLOT(onMessage(Jreen::Message,bool)));
+	connect(d->room, SIGNAL(serviceMessageReceived(Jreen::Message)),
+			this, SLOT(onServiceMessage(Jreen::Message)));
 	connect(d->room, SIGNAL(subjectChanged(QString,QString)),
 			this, SLOT(onSubjectChanged(QString,QString)));
 	connect(d->room, SIGNAL(leaved()), this, SIGNAL(left()));
 	connect(d->room, SIGNAL(joined()), this, SIGNAL(joined()));
-	connect(d->room, SIGNAL(error(jreen::Error::Ptr)),
-			this, SLOT(onError(jreen::Error::Ptr)));
+	connect(d->room, SIGNAL(error(Jreen::Error::Ptr)),
+			this, SLOT(onError(Jreen::Error::Ptr)));
 	//	if (!password.isEmpty())
 	//		d->room->setPassword(password);
 	d->isJoined = false;
@@ -126,7 +134,7 @@ JMUCSession::~JMUCSession()
 qutim_sdk_0_3::Buddy *JMUCSession::me() const
 {
 	Q_D(const JMUCSession);
-	return d->users.value(d->nick);
+	return d->users.value(d->room->nick());
 }
 
 ChatUnit *JMUCSession::participant(const QString &nick)
@@ -134,6 +142,19 @@ ChatUnit *JMUCSession::participant(const QString &nick)
 	return d_func()->users.value(nick);
 }
 
+QString JMUCSession::nick()
+{
+	return d_func()->room->nick();
+}
+
+void JMUCSession::setNick(const QString &nick)
+{
+	Q_D(JMUCSession);
+	d->room->setNick(nick);
+	if (!d->room->isJoined())
+		emit nickChanged(nick);
+}
+
 void JMUCSession::join()
 {
 	Q_D(JMUCSession);
@@ -142,29 +163,6 @@ void JMUCSession::join()
 	d->isJoined = true;
 	d->room->join();
 	setChatState(ChatStateActive);
-	//		Q_D(JMUCSession);
-	//		Presence &pres = d->account->client()->presence();
-	//		d->isAutoRejoin = false;
-	//		if (d->isJoined) {
-	//			d->room->setPresence(pres.subtype(), pres.status());
-	//			d->users.value(d->nick)->setStatus(pres.subtype(), pres.priority());
-	//		} else {
-	//			ChatSession *session = ChatLayer::get(this, false);
-	//			if (session) {
-	//				foreach (JMUCUser *muc, d->users.values()) {
-	//					session->removeContact(muc);
-	//					muc->deleteLater();
-	//				}
-	//			}
-	//			d->users.clear();
-	//			d->messages.clear();
-	//			if (d->lastMessage.isValid())
-	//				d->room->setRequestHistory(d->lastMessage.toUTC()
-	//						.toString("yyyy-MM-ddThh:mm:ss.zzzZ").toStdString());
-	////			uncomment for perfomance testing
-	////			d->room->setRequestHistory(0,MUCRoom::HistoryMaxStanzas);
-	//			d->room->join(pres.subtype(), pres.status(), pres.priority());
-	//		}
 }
 
 void JMUCSession::leave()
@@ -175,6 +173,59 @@ void JMUCSession::leave()
 	d->room->leave();
 	d->isJoined = false;
 	setChatState(ChatStateGone);
+	ChatSession *session = ChatLayer::get(this, false);
+	//remove users
+	foreach (QString name, d->users.keys()) {
+		JMUCUser *user = d->users.take(name);
+		if (session)
+			session->removeContact(user);
+		user->deleteLater();
+	}
+}
+
+void JMUCSession::kick(const QString &nick, const QString &reason)
+{
+	d_func()->room->kick(nick, reason);
+}
+
+void JMUCSession::ban(const QString &nick, const QString &reason)
+{
+	d_func()->room->ban(nick, reason);
+}
+
+void JMUCSession::unban(const QString &jid, const QString &reason)
+{
+	d_func()->room->setAffiliation(JID(jid), MUCRoom::AffiliationNone, reason);
+}
+
+void JMUCSession::member(const QString &nick, const QString &reason)
+{
+	d_func()->room->setAffiliation(nick, MUCRoom::AffiliationMember, reason);
+}
+
+void JMUCSession::voice(const QString &nick, const QString &reason)
+{
+	d_func()->room->setRole(nick, MUCRoom::RoleParticipant, reason);
+}
+
+void JMUCSession::moder(const QString &nick, const QString &reason)
+{
+	d_func()->room->setRole(nick, MUCRoom::RoleModerator, reason);
+}
+
+void JMUCSession::visitor(const QString &nick, const QString &reason)
+{
+	d_func()->room->setRole(nick, MUCRoom::RoleVisitor, reason);
+}
+
+void JMUCSession::admin(const QString &nick, const QString &reason)
+{
+	d_func()->room->setAffiliation(nick, MUCRoom::AffiliationAdmin, reason);
+}
+
+void JMUCSession::owner(const QString &nick, const QString &reason)
+{
+	d_func()->room->setAffiliation(nick, MUCRoom::AffiliationOwner, reason);
 }
 
 bool JMUCSession::isJoined()
@@ -191,9 +242,13 @@ QString JMUCSession::id() const
 bool JMUCSession::sendMessage(const qutim_sdk_0_3::Message &message)
 {
 	Q_D(JMUCSession);
+	qDebug() << d->jid;
+	qDebug() << d->jid.bareJID();
 
 	if (account()->status() == Status::Offline)
 		return false;
+	if (!isJoined())
+		return false;
 
 	if (message.text().startsWith("/nick ")) {
 		QString nick = message.text().section(' ', 1);
@@ -210,30 +265,31 @@ bool JMUCSession::sendMessage(const qutim_sdk_0_3::Message &message)
 			return true;
 		}
 	}
-	jreen::Message jMsg(jreen::Message::Groupchat, message.chatUnit()->id(), message.text());
+	Jreen::Message jMsg(Jreen::Message::Groupchat, d->jid.bareJID(), message.text());
 	jMsg.setID(d->account->client()->getID());
 	d->messages.insert(jMsg.id(), message.id());
 	d->account->client()->send(jMsg);
 	return true;
 }
 
-bool JMUCSession::sendPrivateMessage(const qutim_sdk_0_3::Message &message)
+bool JMUCSession::sendPrivateMessage(const QString &id, const qutim_sdk_0_3::Message &message)
 {
 	Q_D(JMUCSession);
 	if (account()->status() == Status::Offline)
 		return false;
-	jreen::Message jMsg(jreen::Message::Chat, message.chatUnit()->id(), message.text());
+	Jreen::Message jMsg(Jreen::Message::Chat, id, message.text());
 	jMsg.setID(d->account->client()->getID());
 	d->account->client()->send(jMsg);
 	return true;
 }
 
-void JMUCSession::onParticipantPresence(const jreen::Presence &presence,
-										const jreen::MUCRoom::Participant *participant)
+void JMUCSession::onParticipantPresence(const Jreen::Presence &presence,
+										const Jreen::MUCRoom::Participant *participant)
 {
 	Q_D(JMUCSession);
+	Notification::Type notificationType = Notification::System;
 	QString nick = presence.from().resource();
-	bool isSelf = nick == d->nick;
+	bool isSelf = nick == d->room->nick();
 	QString text;
 	if (participant->isBanned() || participant->isKicked()) {
 		QString reason = participant->reason();
@@ -256,10 +312,15 @@ void JMUCSession::onParticipantPresence(const jreen::Presence &presence,
 			}
 		} else {
 			JMUCUser *user = d->users.take(nick);
-			if (ChatSession *session = ChatLayer::get(this, false))
+			if (ChatSession *session = ChatLayer::get(this, false)) {
 				session->removeContact(user);
-			user->deleteLater();
+			}
+			if (ChatSession *session = ChatLayer::get(user, false)) {
+				user->setParent(session);
+			} else
+				user->deleteLater();
 		}
+		notificationType = Notification::ChatUserLeft;
 	} else if (participant->isNickChanged()) {
 		QString newNick = participant->newNick();
 		if (newNick.isEmpty())
@@ -275,7 +336,7 @@ void JMUCSession::onParticipantPresence(const jreen::Presence &presence,
 		//			if (session)
 		//				session->session()->setResource(participant.newNick);
 		if (isSelf) {
-			d->nick = newNick;
+			emit nickChanged(newNick);
 			emit meChanged(me());
 		}
 		user->setStatus(presence);
@@ -306,6 +367,7 @@ void JMUCSession::onParticipantPresence(const jreen::Presence &presence,
 			d->users.insert(nick, user);
 			if (ChatSession *session = ChatLayer::get(this, false))
 				session->addContact(user);
+			notificationType = Notification::ChatUserJoined;
 		} else if (!user) {
 			return;
 		} else if (presence.subtype() == Presence::Unavailable) {
@@ -314,6 +376,7 @@ void JMUCSession::onParticipantPresence(const jreen::Presence &presence,
 			if (ChatSession *session = ChatLayer::get(this, false))
 				session->removeContact(user);
 			user->deleteLater();
+			notificationType = Notification::ChatUserLeft;
 		} else {
 			user->setStatus(presence);
 		}
@@ -346,18 +409,18 @@ void JMUCSession::onParticipantPresence(const jreen::Presence &presence,
 			user->setMUCAffiliation(participant->affiliation());
 			user->setMUCRole(participant->role());
 		}
-		if (presence.subtype() != Presence::Unavailable && !presence.error()) {
-			VCardUpdate::Ptr vcard = presence.findExtension<VCardUpdate>();
-			if(vcard) {
-				QString hash = vcard->photoHash();
-				if (user->avatarHash() != hash) {
-					if(hash.isEmpty() || QFile(d->account->getAvatarPath() % QLatin1Char('/') % hash).exists())
-						user->setAvatar(hash);
-					//					else if (d->avatarsAutoLoad)
-					//						d->account->vCardManager()->fetchVCard(user->id());
-				}
-			}
-		}
+//		if (presence.subtype() != Presence::Unavailable && !presence.error()) {
+//			VCardUpdate::Ptr vcard = presence.payload<VCardUpdate>();
+//			if(vcard && vcard->hasPhotoInfo()) {
+//				QString hash = vcard->photoHash();
+//				if (user->avatarHash() != hash) {
+//					if(hash.isEmpty() || QFile(d->account->getAvatarPath() % QLatin1Char('/') % hash).exists())
+//						user->setAvatar(hash);
+//					else if (d->avatarsAutoLoad)
+//						d->account->vCardManager()->fetchVCard(user->id());
+//				}
+//			}
+//		}
 		//WTF?  Oo
 		//if (!d->isJoined && isSelf) {
 		//	d->isJoined = true;
@@ -365,17 +428,15 @@ void JMUCSession::onParticipantPresence(const jreen::Presence &presence,
 		//}
 	}
 	if (!text.isEmpty() && (d->isJoined || participant->isKicked() || participant->isBanned())) {
-		qutim_sdk_0_3::Message msg(text);
-		msg.setChatUnit(this);
-		msg.setTime(QDateTime::currentDateTime());
-		msg.setProperty("service", true);
-		msg.setProperty("silent", true);
-		if (ChatSession *chatSession = ChatLayer::get(this, false))
-			chatSession->appendMessage(msg);
+		NotificationRequest request(notificationType);
+		request.setObject(this);
+		request.setText(text);
+		request.setProperty("senderName", nick);
+		request.send();
 	}
 }
 
-void JMUCSession::onMessage(const jreen::Message &msg, bool priv)
+void JMUCSession::onMessage(const Jreen::Message &msg, bool priv)
 {
 	Q_D(JMUCSession);
 	//	Q_ASSERT(room == d->room);
@@ -392,7 +453,7 @@ void JMUCSession::onMessage(const jreen::Message &msg, bool priv)
 			return;
 		qutim_sdk_0_3::Message coreMsg(msg.body());
 		coreMsg.setChatUnit(user);
-		coreMsg.setIncoming(msg.from().resource() != d->room->nick());
+		coreMsg.setIncoming(true); // msg.from().resource() != d->room->nick());
 		ChatSession *chatSession = ChatLayer::get(user, true);
 		chatSession->appendMessage(coreMsg);
 	} else {
@@ -402,11 +463,9 @@ void JMUCSession::onMessage(const jreen::Message &msg, bool priv)
 		coreMsg.setProperty("senderName", nick);
 		if (user)
 			coreMsg.setProperty("senderId", user->id());
-		if (!coreMsg.text().contains(d->room->nick()))
-			coreMsg.setProperty("silent", true);
 		coreMsg.setIncoming(msg.from().resource() != d->room->nick());
 		ChatSession *chatSession = ChatLayer::get(this, true);
-		const DelayedDelivery *when = msg.when();
+		DelayedDelivery::Ptr when = msg.when();
 		if (when) {
 			coreMsg.setProperty("history", true);
 			coreMsg.setTime(when->dateTime());
@@ -427,13 +486,18 @@ void JMUCSession::onMessage(const jreen::Message &msg, bool priv)
 	}
 }
 
-void JMUCSession::onServiceMessage(const jreen::Message &msg)
+void JMUCSession::onServiceMessage(const Jreen::Message &msg)
 {
 	//TODO add capthca handler
+	Q_D(JMUCSession);
+	if (!msg.subject().isEmpty())
+		d->topic = msg.subject();
 	ChatSession *chatSession = ChatLayer::get(this, true);
 	qutim_sdk_0_3::Message coreMsg(msg.body());
 	coreMsg.setChatUnit(this);
 	coreMsg.setProperty("service",true);
+	coreMsg.setProperty("silent", true);
+	coreMsg.setIncoming(true);
 	chatSession->appendMessage(coreMsg);
 }
 
@@ -451,10 +515,13 @@ void JMUCSession::onSubjectChanged(const QString &subject, const QString &nick)
 {
 	Q_UNUSED(nick);
 	Q_D(JMUCSession);
+	if (d->topic == subject)
+		return;
 	qutim_sdk_0_3::Message msg(tr("Subject:") % "\n" % subject);
 	msg.setChatUnit(this);
 	msg.setTime(QDateTime::currentDateTime());
 	msg.setProperty("service", true);
+	msg.setIncoming(true);
 	if (ChatSession *chatSession = ChatLayer::get(this, false))
 		chatSession->appendMessage(msg);
 	setConferenceTopic(subject);
@@ -540,7 +607,7 @@ void JMUCSession::onSubjectChanged(const QString &subject, const QString &nick)
 //	Q_ASSERT(room == d_func()->room);
 //}
 
-void JMUCSession::setBookmark(const jreen::Bookmark::Conference &bookmark)
+void JMUCSession::setBookmark(const Jreen::Bookmark::Conference &bookmark)
 {
 	Q_D(JMUCSession);
 	d->bookmark = bookmark;
@@ -551,9 +618,11 @@ void JMUCSession::setBookmark(const jreen::Bookmark::Conference &bookmark)
 		d->title = id();
 	if (d->title != previous)
 		emit titleChanged(d->title, previous);
+
+	emit bookmarkChanged(bookmark);
 }
 
-jreen::Bookmark::Conference JMUCSession::bookmark()
+Jreen::Bookmark::Conference JMUCSession::bookmark()
 {
 	return d_func()->bookmark;
 }
@@ -565,12 +634,12 @@ QString JMUCSession::title() const
 
 void JMUCSession::showConfigDialog()
 {
-	//	d_func()->isConfiguring = true;
-	if (d_func()->config)
-		return;
-	d_func()->config = new JConferenceConfig(d_func()->room);
-	//	connect(dialog, SIGNAL(destroyDialog()), SLOT(closeConfigDialog()));
-	d_func()->config->show();
+//	//	d_func()->isConfiguring = true;
+//	if (d_func()->config)
+//		return;
+//	d_func()->config = new JConferenceConfig(d_func()->room);
+//	//	connect(dialog, SIGNAL(destroyDialog()), SLOT(closeConfigDialog()));
+//	d_func()->config->show();
 }
 
 void JMUCSession::closeConfigDialog()
@@ -580,10 +649,14 @@ void JMUCSession::closeConfigDialog()
 
 bool JMUCSession::enabledConfiguring()
 {
+	//TODO add signal configuring changed
 	Q_D(JMUCSession);
-	if (JMUCUser *i = d->users.value(d->nick))
-		return i->affiliation() == MUCRoom::AffiliationOwner && !d->config;
-	return false;
+	if(!isJoined())
+		return false;
+
+	MUCRoom::Affiliation affiliation = d->room->affiliation();
+	return affiliation == MUCRoom::AffiliationOwner
+	        || affiliation == MUCRoom::AffiliationAdmin;
 }
 
 void JMUCSession::loadSettings()
@@ -601,7 +674,7 @@ bool JMUCSession::isError()
 	return d_func()->isError;
 }
 
-jreen::MUCRoom *JMUCSession::room()
+Jreen::MUCRoom *JMUCSession::room()
 {
 	return d_func()->room;
 }
@@ -616,18 +689,19 @@ ChatUnitList JMUCSession::lowerUnits()
 
 QString JMUCSession::topic() const
 {
-	return d_func()->room->subject();
+	return d_func()->topic;
 }
 
 void JMUCSession::setTopic(const QString &topic)
 {
-	//	setConferenceTopic(topic);
 	d_func()->room->setSubject(topic);
 }
 
 void JMUCSession::setConferenceTopic(const QString &topic)
 {
 	Q_D(JMUCSession);
+	if (d->topic == topic)
+		return;
 	QString previous = d->topic;
 	d->topic = topic;
 	emit topicChanged(topic, previous);
@@ -645,29 +719,52 @@ void JMUCSession::setAutoJoin(bool join)
 
 void JMUCSession::invite(qutim_sdk_0_3::Contact *contact, const QString &reason)
 {
-	//	d_func()->room->invite(contact->id().toStdString(), reason.toStdString());
+	d_func()->room->invite(contact->id(), reason);
 }
 
-void JMUCSession::onError(jreen::Error::Ptr error)
+void JMUCSession::onError(Jreen::Error::Ptr error)
 {
 	Q_D(JMUCSession);
 	debug() << "error" << error->condition();
-	if(error->condition() == Error::Conflict) {
-		Notifications::send(Notifications::System,
-							this,
-							QT_TRANSLATE_NOOP("Jabber","You already in conference with another nick"));
+	if (error->condition() == Error::Conflict) {
+		QString message = QCoreApplication::translate("Jabber", "You are already in conference with another nick");
+		NotificationRequest request(Notification::System);
+		request.setObject(this);
+		request.setText(message);
+		request.send();
+		QString resource = d->account->client()->jid().resource();
+		if (!d->room->nick().endsWith(resource)) {
+			QString nick = d->room->nick();
+			nick += QLatin1Char('/');
+			nick += resource;
+			onNickSelected(nick);
+			return;
+		}
 
-		QString nick = QInputDialog::getText(QApplication::activeWindow(),
-											 QT_TRANSLATE_NOOP("Jabber","You already in conference with another nick"),
-											 QT_TRANSLATE_NOOP("Jabber","Please select another nickname"));
-		////TODO add regexp
-		if(nick.isEmpty())
-			leave();
-		d->room->setNick(nick);
-		d->room->join();
-	} else if(error->condition() == Error::Forbidden) {
+		QInputDialog *dialog = new QInputDialog(QApplication::activeWindow());
+		dialog->setWindowTitle(message);
+	    dialog->setLabelText(QCoreApplication::translate("Jabber", "Please select another nickname"));
+	    dialog->setTextValue(d->room->nick());
+		dialog->show();
+		connect(dialog, SIGNAL(textValueSelected(QString)), SLOT(onNickSelected(QString)));
+		connect(dialog, SIGNAL(finished(int)), dialog, SLOT(deleteLater()));
+	} else if (error->condition() == Error::Forbidden) {
 		leave();
 	}
 }
 
+void JMUCSession::onNickSelected(const QString &nick)
+{
+	////TODO add regexp
+	if(nick.isEmpty()) {
+		leave();
+	} else {
+		Q_D(JMUCSession);
+		leave();
+		d->room->setNick(nick);
+		join();
+	}
 }
+
+}
+
